#include <smlib>

#if defined _smlib_client_included
#endinput
#endif
#define _smlib_client_included



// Hud Element hiding flags
#define HIDEHUD_WEAPONSELECTION		( 1<<0 )	// Hide ammo count & weapon selection
#define HIDEHUD_FLASHLIGHT			( 1<<1 )
#define HIDEHUD_ALL					( 1<<2 )
#define HIDEHUD_HEALTH				( 1<<3 )	// Hide health & armor / suit battery
#define HIDEHUD_PLAYERDEAD			( 1<<4 )	// Hide when local player's dead
#define HIDEHUD_NEEDSUIT			( 1<<5 )	// Hide when the local player doesn't have the HEV suit
#define HIDEHUD_MISCSTATUS			( 1<<6 )	// Hide miscellaneous status elements (trains, pickup history, death notices, etc)
#define HIDEHUD_CHAT				( 1<<7 )	// Hide all communication elements (saytext, voice icon, etc)
#define HIDEHUD_CROSSHAIR			( 1<<8 )	// Hide crosshairs
#define HIDEHUD_VEHICLE_CROSSHAIR	( 1<<9 )	// Hide vehicle crosshair
#define HIDEHUD_INVEHICLE			( 1<<10 )

/**
* Sets the Hud flags of a client
*
* @param client		Client index.
* @param flags			Flag to set, use one of the HIDEHUD_ hiding constants
* @noreturn
*/
stock Client_SetHud(client, flags) {
	
	SetEntProp(client, Prop_Send, "m_iHideHUD", flags);
}

/**
* Set a client's Armor
*
* @param client		Client index.
* @param value			armor to set (anything above 511 will overload), not supported by all games
* @noreturn
*/
stock Client_SetArmor(client,value){
	
	SetEntProp(client, Prop_Data, "m_ArmorValue", value, 1);
	return value;
}

/**
* Set a client's Frag points
*
* @param client		Client index.
* @param value			frag points to set
* @noreturn
*/
stock Client_SetFrags(client, value) {
	
	SetEntProp(client, Prop_Data, "m_iFrags", value);
}

/**
* Sets a client's Death points
*
* @param client		Client index.
* @param value			death points to set
* @noreturn
*/
stock Client_SetDeaths(client, value) {
	
	SetEntProp(client, Prop_Data, "m_iDeaths", value);
}

/**
* Checks if an entity index is a player and ingame
*
* @param entity		An entity index
* @return				true if the specified entity index is a player ingame, false otherwise
*/
stock bool:Client_IsPlayer(entity) {
	
	if (entity > GetMaxEntities()) {
		entity = EntRefToEntIndex(entity);
	}
	
	if (entity >= 1 && entity <= MaxClients) {
		if (IsValidEntity(entity) && IsClientInGame(entity)) {
			return true;
		}
	}
	
	return false;
}

/**
* Finds a player by SteamID
*
* @param auth			SteamID to search for
* @return				true if the specified entity index is a player ingame, false otherwise
*/
stock Client_GetBySteamId(const String:auth[]) {
	
	new String:inGameAuth[STEAMAUTH_LENGTH];
	for (new client=1; client<=MaxClients; client++) {
		
		if (!IsClientConnected(client) || !IsClientAuthorized(client)) {
			continue;
		}
		
		GetClientAuthString(client, inGameAuth, STEAMAUTH_LENGTH);
		if (StrEqual(auth,inGameAuth, false)) {
			return client;
		}
	}
	
	return -1;
}

/**
* Puts the specified client into thirdperson or back to firstperson when false
* This doesn't work in all games, it works in CS:S and DOD:S and some other games
*
* @param				SteamID to search for
* @return				true if the specified entity index is a player ingame, false otherwise
*/
stock Client_SetThirdPersonMode(client, enable=true) {
	
	if(enable){
		SetEntPropEnt(client, Prop_Send, "m_hObserverTarget", 0); 
		SetEntProp(client, Prop_Send, "m_iObserverMode", 1);
		SetEntProp(client, Prop_Send, "m_bDrawViewmodel", 0);
		SetEntProp(client, Prop_Send, "m_iFOV", 120);
	}
	else {
		SetEntPropEnt(client, Prop_Send, "m_hObserverTarget", -1);
		SetEntProp(client, Prop_Send, "m_iObserverMode", 0);
		SetEntProp(client, Prop_Send, "m_bDrawViewmodel", 1);
		SetEntProp(client, Prop_Send, "m_iFOV", 90);
	}
}

/**
* Returns if the client is in thirdperson mode or not
*
* @param client		client index
* @return				true if the client is currently in thirdperson mode, false otherwise
*/
stock Client_IsInThirdPersonMode(client) {
	
	return GetEntProp(client, Prop_Data, "m_iObserverMode");
}

/**
* Sets the client maxspeed to the given value (in units per second)
*
* @param client		client index
* @param maxspeed		the maximum speed the client can walk
* @noreturn
*/
stock Client_SetSpeed(client, Float:maxspeed) {
	
	SetEntPropFloat(client, Prop_Data, "m_flMaxspeed", maxspeed);
	
	//PrintToServer("[SMLIB-Debug] Speed of client %d is now: %f",client,GetEntPropFloat(client,Prop_Data,"m_flMaxspeed"));
}

#define FFADE_IN			0x0001		// Just here so we don't pass 0 into the function
#define FFADE_OUT			0x0002		// Fade out (not in)
#define FFADE_MODULATE		0x0004		// Modulate (don't blend)
#define FFADE_STAYOUT		0x0008		// ignores the duration, stays faded out until new ScreenFade message received
#define FFADE_PURGE			0x0010		// Purges all other fades, replacing them with this one

/**
* Fades a client's screen to a specified color
*
* @param client		player for which to fade the screen
* @param duration		duration in seconds the effect stays
* @param mode			fade mode, see FFADE_ defines
* @param holdtime		holdtime in seconds
* @param r				red amount
* @param g				green amount
* @param b				blue amount
* @param a				transparency
* @noreturn
*/
stock Client_FadeScreen(client, duration, mode, holdtime=-1, r=0, g=0, b=0, a=255) {
	
	new Handle:msg = StartMessageOne("Fade", client);
	BfWriteShort(msg, duration);	// Fade duration
	BfWriteShort(msg, holdtime);	// Fade hold time
	BfWriteShort(msg, mode);		// What to do
	BfWriteByte(msg, r);			// Color R
	BfWriteByte(msg, g);			// Color G
	BfWriteByte(msg, b);			// Color B
	BfWriteByte(msg, a);			// Color Alpha
	EndMessage();
}
/**
* Fades client to black
*
* @param client		client index
* @param alphaAmount	how much its faded to black
* @noreturn
*/
public Client_FadeToBlack(client, alphaAmount){
	
	new UserMsg:fade = GetUserMessageId("Fade");
	
	if((fade == UserMsg:-1) || (fade == INVALID_MESSAGE_ID)){
		return;
	}
	
	new targets[2];
	targets[0] = client;
	
	new Handle:message = StartMessageEx(fade,targets,1);
	BfWriteShort(message, 1536);
	BfWriteShort(message, 1536);
	
	if (alphaAmount == 0){
		
		BfWriteShort(message, (0x0001 | 0x0010));
	}
	else{
		
		BfWriteShort(message, (0x0002 | 0x0008));
	}
	
	BfWriteByte(message, 0);
	BfWriteByte(message, 0);
	BfWriteByte(message, 0);
	BfWriteByte(message, alphaAmount);
	
	EndMessage();
}

/**
* This function retrieves an array that holds all clones of a client by IP check.
*
* @param client		Client index.
* @param closelist		An array that holds all clones of a client.
* @return				Returns how many clones a client has.
*/
stock Client_GetClones(client,clonelist[]){
	
	decl String:clientip[17], String:cloneip[17];
	new i=0;
	
	for (new clone=1; clone<=MaxClients; clone++) {
		
		if(IsClientInGame(clone)){
			
			GetClientIP(client,clientip,17);
			GetClientIP(clone,cloneip,17);
			
			if(StrEqual(clientip,cloneip,false)){
				
				clonelist[i] = client;
				i++;
			}
		}
	}
	return i;
}

/*
* This function returns true if the client is at a ladder..
*
* @param client			Client index.
* @return				Returns true if the client is on a ladder other wise false.
*/
stock bool:InLadder(client){
	
	new MoveType:movetype = GetEntityMoveType(client);
	
	if (movetype == MOVETYPE_LADDER){
		
		return true;
	}
	else{
		
		return false;
	}
}


	
#define WATER_LEVEL_NOT_IN_WATER 	0
#define WATER_LEVEL_FEET_IN_WATER 	1
#define WATER_LEVEL_WAIST_IN_WATER	2
#define WATER_LEVEL_HEAD_IN_WATER	3

/*
* This function returns how deep a client is in water. enum Water_Level
*
* @param client			Client index.
* @return				Returns 0 if not in water. 1 if feets are in water. 2 if waist is in water. 3 if head is in water.
*/
stock Water_Level:Client_GetWaterLevel(client){
	
	return Water_Level:GetEntProp(client, Prop_Send, "m_nWaterLevel");
}

/*
* Returns how much suit sprint power a client has left in percent.
*
* @param client			Client index.
* @return				returns the actual power left in percent.
*/
stock Float:Client_GetSuitSprintPower(client){

	return GetEntPropFloat(client, Prop_Send, "m_flSuitPower");
}

/*
* Sets a client suit sprint power in percent.
*
* @param client			Client index.
* @param power			power (0.0 to 100.0)
* @noreturn
*/
stock Client_SetSuitSprintPower(client,Float:power){
	
	SetEntPropFloat(client, Prop_Send, "m_flSuitPower", power);
}

/*
* Returns the client count put in the server. 
*
* @param inGameOnly		If false connecting players are also counted.
* @param countBots		If true bots will be counted too.
* @return				Client count in the server.
*/
stock Clients_GetCount(bool:inGameOnly=true,bool:countBots=false){
	new clients = 0;
	
	for(new client=1;client<=GetMaxClients();client++){
		
		if((inGameOnly ? IsClientInGame(client) : IsClientConnected(client)) && (countBots ? true : !IsFakeClient(client))){
			
			clients++;
		}
	}
	return clients;
}
