/*****************************************************************


		SMLIB


*****************************************************************/
#include <smlib>
/*****************************************************************


		D O U B L E   I N C L U D E   P R E V E N T I O N


*****************************************************************/
#if defined _smlib_entities_included
  #endinput
#endif
#define _smlib_entities_included

/*****************************************************************


		D E F I N E S


*****************************************************************/


/*****************************************************************


		G L O B A L   V A R S


*****************************************************************/


/*****************************************************************


		F U N C T I O N S


*****************************************************************/
/*
 * Finds an entity by targetname (m_iName).
 *
 * @param name			Targetname of the entity you want so search.
 * @param class			If you know the classname of the entity or want to search only the targetname of a specific class then enter it here.
 * @noreturn
 */
stock Entity_FindByName(const String:name[], const String:class[] = "") {
	
	decl String:entityName[64];
	
	if (StrEqual(class, "")) {

		new maxEntities = GetMaxEntities();
		for (new entity=0; entity<maxEntities; entity++) {
			
			if (!IsValidEntity(entity)) {
				continue;
			}
			
			Entity_GetName(entity, entityName, sizeof(entityName));
			
			if (StrEqual(entityName, name)) {
				return entity;
			}
		}
	}
	else {
		
		new entity = -1;
		while ((entity = FindEntityByClassname(entity, class)) != -1) {
			
			Entity_GetName(entity, entityName, sizeof(entityName));
			
			if (StrEqual(entityName, name)) {
				return entity;
			}
		}
	}
	
	return -1;
}
/**
 * Gets the Name of an entity.
 *
 * @param entity			Entity index.
 * @param buffer			Return/Output buffer.
 * @param size				Max size of buffer.
 * @noreturn
 */
stock Entity_GetName(entity, String:buffer[], size) {

	GetEntPropString(entity, Prop_Data, "m_iName", buffer, size);
}
/**
 * Sets the Name of an entity.
 *
 * @param entity			Entity index.
 * @param name			The target name you want to give.
 * @noreturn
 */
stock Entity_SetName(entity, String:name[], any:...) {
	
	decl String:vformat[128];
	VFormat(vformat, sizeof(vformat), name, 3);

	SetEntPropString(entity, Prop_Data, "m_iName", vformat);
}

/**
 * Gets the Origin of an entity.
 *
 * @param entity			Entity index.
 * @param vec				return vector.
 * @noreturn
 */
stock Entity_GetAbsOrigin(entity, Float:vec[3]) {
	
	GetEntPropVector(entity, Prop_Send, "m_vecOrigin", vec);
}
/**
 * Gets the Angles of an entity
 *
 * @param entity			Entity index.
 * @param vec				return vector.
 * @noreturn
 */
stock Entity_GetAbsAngles(entity, Float:angles[3]) {
	GetEntPropVector(entity, Prop_Send, "m_angAbsRotation", angles);
}
/**
 * Searches for the closest entity in relation to the given entity.
 *
 * @param entity			Entity index to start the search
 * @param only_clients		True if you only want to search for clients
 * @return					The closest entity.
 */
stock Entity_GetClosestEntity(startentity, bool:only_clients=false) {
	
	decl Float:vec_start[3], Float:vec_end[3];
	new Float:distance = 0.0;
	new closestEnt = -1;
	
	if (GetEntSendPropOffs(startentity, "m_vecOrigin") == -1) {
		return -1;
	}
	
	GetEntPropVector(startentity, Prop_Send, "m_vecOrigin", vec_start);
	
	new maxents = -1;
	
	if(only_clients){
		maxents = MaxClients;
	}
	else {
		maxents = GetMaxEntities();
	}
	
	for (new entity=1;entity<=maxents;entity++) {
		
		if(IsValidEntity(entity) && entity != startentity) {
			
			if(GetEntSendPropOffs(entity, "m_vecOrigin") == -1){
				continue;
			}
			
			GetEntPropVector(entity, Prop_Send, "m_vecOrigin", vec_end);
			
			new Float:dist = GetVectorDistance(vec_start, vec_end);
			
			if (distance == 0.0 || dist < distance) {
				distance = dist;
				closestEnt = entity;
			}
		}
	}
	
	return closestEnt;
}
/**
 * Gets the Mins of an entity.
 *
 * @param entity			Entity index
 * @param vec				return vector.
 * @noreturn
 */
stock Entity_GetMaxs(entity, Float:vec[3]){
	GetEntPropVector(entity, Prop_Send, "m_vecMaxs", vec);	
}
/**
 * Gets the Mins of an entity.
 *
 * @param entity			entity index.
 * @param vec				return vector.
 * @noreturn
 */
stock Entity_GetMins(entity, Float:vec[3]){
	GetEntPropVector(entity, Prop_Send, "m_vecMins", vec);	
}

/**
 * Sets the Maxs of an entity.
 *
 * @param entity			entity index.
 * @param vec				set vector.
 * @noreturn
 */
stock Entity_SetMaxs(entity, Float:vec[3]){
	SetEntPropVector(entity, Prop_Send, "m_vecMaxs", vec);	
}

/**
 * Sets the Mins of an entity.
 *
 * @param entity			entity index.
 * @param vec				set vector.
 * @noreturn
 */
stock Entity_SetMins(entity, Float:vec[3]){
	SetEntPropVector(entity, Prop_Send, "m_vecMins", vec);	
}
/**
 * Retrieves the model path of a given entity
 *
 * @param entity		entity reference or index
 * @param model			buffer String for the model
 * @param size			max size of buffer string
 * @noreturn				
 */
stock Entity_GetModel(entity, String:buffer[], size) {
	
	GetEntPropString(entity, Prop_Data, "m_ModelName", buffer, size);
}
/**
 * Sets the collision group of an entity.
 *
 * @param entity		entity index
 * @param value			the new collision group.
 * @noreturn
 */
stock Entity_SetCollisionGroup(entity,any:value){
	SetEntProp(entity, Prop_Data, "m_CollisionGroup", value);
}
/**
 * Gets the collision group of an entity.
 *
 * @param entity		entity index
 * @return				Entity collision group.
 */
stock Entity_GetCollisionGroup(entity){
	return GetEntProp(entity, Prop_Data, "m_CollisionGroup");
}
/**
 * Sets the solidity of a client against other clients
 *
 * @param client		client index
 * @param solid			should the client be solid to other players (false means he is not solid to other players)
 * @noreturn
 */
stock Entity_SetSolidity(client,bool:solid=true){
	
	if(solid){
		Entity_SetCollisionGroup(client,5);
	}
	else {
		Entity_SetCollisionGroup(client,2);
	}
}
/**
 * Returns if an entity is solid against an other entity.
 *
 * @param entity		The first entity.
 * @param targetentity	The second entity. If left empty (-1) then the first entity is only checked if its in collision group 2 (blocks players).
 * @return				True if the two entitys can colide otherwise false.
 */
stock bool:Entity_CanCollide(entity,targetentity=-1){
	
	if(targetentity == -1){
		
		if(Entity_GetCollisionGroup(entity) == 2){
			return true;
		}
		
		return false;
	}
	else {
		
		if(Entity_GetCollisionGroup(entity) == Entity_GetCollisionGroup(targetentity)){
			return true;
		}
		
		return false;
	}
}

/**
 * Gets the velocity of an entity.
 *
 * @param entity		Entity index.
 * @param vel			An 3 dim array that holds the velocity.
 * @noreturn
 */
stock Entity_GetAbsVelocity(entity,Float:vel[3]){
	
	GetEntPropVector(entity, Prop_Data, "m_vecVelocity", vel);
}

/**
 * Sets the velocity of an entity.
 *
 * @param entity		Entity index.
 * @param vel			An 3 dim array that holds the velocity.
 * @noreturn
 */
stock Entity_SetAbsVelocity(entity,Float:vel[3]){
	
	SetEntPropVector(entity, Prop_Data, "m_vecVelocity", vel);
}

/**
 * Returns true if the entity is locked.
 *
 * @param entity		Entity index.
 * @return				True if locked otherwise false.
 */
stock bool:Entity_IsLocked(entity) {
	
	return bool:GetEntProp(entity, Prop_Data, "m_bLocked", 1);
}

/**
 * Locks an entity.
 *
 * @param entity		Entity index.
 * @noreturn
 */
stock Entity_Lock(entity) {
	SetEntProp(entity, Prop_Data, "m_bLocked", 1, 1);
}
/**
 * Unlocks an entity.
 *
 * @param entity		Entity index.
 * @noreturn
 */
stock Entity_UnLock(entity) {
	SetEntProp(entity, Prop_Data, "m_bLocked", 0, 1);
}

/**
* Set the health points of an entity
*
* @param entity			entity index.
* @param value			health to set (anything above 511 will overload)
* @noreturn
*/
stock Entity_SetHealth(entity, value) {
	
	SetEntProp(entity, Prop_Data, "m_iHealth", value, 1);
	return value;
}

/**
* Get the health points of an entity
*
* @param entity			entity index.
* @return				current health points
*/
stock Entity_GetHealth(entity) {
	
	return GetEntProp(entity, Prop_Data, "m_iHealth", 1);
}

/**
* Set the max health points of an entity
*
* @param entity			entity index.
* @param value			max health to set (anything above 511 will overload)
* @noreturn
*/
stock Entity_SetMaxHealth(entity, value) {
	
	SetEntProp(entity, Prop_Data, "m_iMaxHealth", value, 1);
	return value;
}

/**
* Get the max health points of an entity
*
* @param entity			entity index.
* @return				max health points
*/
stock Entity_GetMaxHealth(entity) {
	
	return GetEntProp(entity, Prop_Data, "m_iMaxHealth", 1);
}

/**
* Add hp to the current health points of an entity
*
* @param entity			entity index.
* @param value			health to add
* @return				returns new health points.
*/
stock Entity_AddHealth(entity, value, maximum=100) {
	
	new entityHP = Entity_GetHealth(entity);
	
	if(entityHP > maximum){
		return entityHP;
	}
	
	value = entityHP+value;
	if(value > maximum){
		Entity_SetHealth(entity,maximum);
		return maximum;
	}
	
	Entity_SetHealth(entity,value);
	return value;
}

/**
* Checks if the given 2 entitys are closer then the given distance.
*
* @param entity			entity index.
* @param target			target entity index.
* @param dist			max float distance value. if -1.0 its always true;
* @return				returns true if the given entities are closer than the given distance value, false otherwise.
*/
stock bool:Entity_InRange(entity,target,Float:dist){
	
	if(dist == -1.0){
		return true;
	}
	
	new Float:entityVec[3];
	new Float:targetVec[3];
	
	Entity_GetAbsOrigin(entity,entityVec);
	Entity_GetAbsOrigin(target,targetVec);
	
	if(GetVectorDistance(entityVec,targetVec) > dist){
		
		return false;
	}
	
	return true;
}

/**
* Function by exvel: Is fail save when used with RemoveEdict.
*
* @param startEnt		The entity index after which to begin searching from. Use -1 to start from the first entity.
* @param classname		Classname of the entity to find.
* @return				Entity index >= 0 if found, -1 otherwise.
*/
stock Entity_FindByClassname(startEnt, const String:classname[]){

	/* If startEnt isn't valid shifting it back to the nearest valid one */

	while (startEnt > -1 && !IsValidEntity(startEnt)) startEnt--;

	return FindEntityByClassname(startEnt, classname);
}
/**
* Freezes the given entity
*
* @param startEnt		Entity index.
* @noreturn
*/
stock Entity_Freeze(entity){
	
	AcceptEntityInput(entity, "disablemotion");
	ChangeEdictState(entity);
}
/**
* Unfreezes the given entity
*
* @param startEnt		Entity index.
* @noreturn
*/
stock Entity_Unfreeze(entity){
	
	AcceptEntityInput(entity, "enablemotion");
	ChangeEdictState(entity);
}

stock Effect_GetColorByHp(health,maxhealth){
	
	new hpcolor = RoundFloat((255.0 / maxhealth) * health);
	if(hpcolor > 255){
		hpcolor = 255;
	}
	if(hpcolor < 1){
		hpcolor = 0;
	}
	return hpcolor;
}