#include <smlib>

#if defined _smlib_math_included
  #endinput
#endif
#define _smlib_math_included


#define ALPHA 0
#define BETA 1
#define GAMMA 2

/**
 * Makes a negative integer number to a positive integer number.
 *
 * @param number		A number that can be positive or negative.
 * @return				Positive number.
 */
stock IntAbs(number){
	
	if(number < 0){
		number *= (-1);
	}
	return number;
}
/**
 * Checks if 2 vectors are equal. If this is used with uneven vector lengths then the equalness will only be checked to the smallest maxlenVecX position.
 *
 * @param vec1			First vector.
 * @param maxlenVec1	Length of the first vector.
 * @param vec2			Second vector.
 * @param maxlenVec2	Length of the second vector.
 * @param tolerance 	If you want to check that those vectors are somewhat even. 0.0 means they are 100% even if this function returns true.
 * @return				True if vectors are equal otherwise false.
 */
stock bool:Math_IsVectorEqual(Float:vec1[],maxlenVec1,Float:vec2[],maxlenVec2,Float:tolerance=0.0){
	
	new found=0;
	new maxlen;
	
	if(maxlenVec1 == maxlenVec2){
		maxlen = maxlenVec1;
	}
	else if(maxlenVec1 > maxlenVec2){
		maxlen = maxlenVec2;
	}
	else {
		maxlen = maxlenVec1;
	}
	
	if(tolerance == 0.0){
		
		for(new i=0;i<maxlen;i++){
			
			if(vec1[i] == vec2[i]){
				
				found++;
			}
		}
	}
	else{
		
		//tolerance can't be negative so we force it to be positive:
		if(tolerance < 0){
			tolerance *= -1;
		}
		
		for(new i=0;i<maxlen;i++){
			
			if((vec1[i] > vec2[i]-tolerance) && (vec1[i] < vec2[i]+tolerance)){
				found++;
			}
			else if((vec2[i] > vec1[i]-tolerance) && (vec2[i] < vec1[i]+tolerance)){
				found++;
			}
		}
	}
	
	if(found == maxlen){
		return true;
	}
	else {
		return false;
	}
}
/**
 * Calculates the vector sum (length of the vector from maps zero point {0.0,0.0,0.0}).
 *
 * @param vec		The vector.
 * @param maxlen	The vector size.
 * @return			The vector sum (length of the vector from maps zero point {0.0,0.0,0.0}).
 */
stock Float:Math_VectorSum(Float:vec[],maxlen){
	
	new Float:scalar=0.0;
	
	for(new i=0;i<maxlen;i++){
		scalar += vec[i]*vec[i];
	}
	
	return SquareRoot(scalar);
}
/**
 * Calculates the alpha,beta,gamma angles of a vector.
 *
 * @param vec			The vector (size must be 3 (x,y,z)).
 * @param anglesOutput	The output of those 3 angles.
 * @param inRadiant		The output is in radian if true.
 * @noreturn
 */
stock Math_DirectionCosine(Float:vec[3],Float:anglesOutput[3],bool:inRadian){
	
	new vecsum = Math_VectorSum(vec,3);
	
	anglesOutput[ALPHA] = ArcCosine((vec[0]/vecsum));
	anglesOutput[BETA] = ArcCosine((vec[1]/vecsum));
	anglesOutput[GAMMA] = ArcCosine((vec[2]/vecsum));
	
	if(!inRadian){
		
		anglesOutput[ALPHA] = RadToDeg(anglesOutput[ALPHA]);
		anglesOutput[BETA] = RadToDeg(anglesOutput[BETA]);
		anglesOutput[GAMMA] = RadToDeg(anglesOutput[GAMMA]);
	}
}
/**
 * Adds a scalar (value) to a vector.
 *
 * @param vec		The vector.
 * @param maxlen	The vector size.
 * @param scalar	The scalar you like to add to every vector component.
 * @noreturn				
 */
stock Math_VectorScalarAddition(Float:vec[],maxlen,Float:scalar){
	
	for(new i=0;i<maxlen;i++){
		
		vec[i] += scalar;
	}
	
}
/**
 * Subtracts a scalar (value) from a vector.
 *
 * @param vec		The vector.
 * @param maxlen	The vector size.
 * @param scalar	The scalar you like to subtract from every vector component.
 * @noreturn				
 */
stock Math_VectorScalarSubtraction(Float:vec[],maxlen,Float:scalar){
	
	Math_VectorScalarAddition(vec,maxlen,(scalar*(-1)));
	
}
/**
 * Multiplicates a scalar (value) to a vector.
 *
 * @param vec		The vector.
 * @param maxlen	The vector size.
 * @param scalar	The scalar you like to multiplicate to every vector component.
 * @noreturn				
 */
stock Math_VectorScalarMultiplication(Float:vec[],maxlen,Float:scalar){
	
	for(new i=0;i<maxlen;i++){
		
		vec[i] *= scalar;
	}
	
}
/**
 * Dividides a scalar (value) from a vector.
 *
 * @param vec		The vector.
 * @param maxlen	The vector size.
 * @param scalar	The scalar you like to dividie from every vector component.
 * @noreturn			
 */
stock Math_VectorScalarDivision(Float:vec[],maxlen,Float:scalar){
	
	Math_VectorScalarMultiplication(vec,maxlen,(Pow(scalar,-1)));
}

